@@ -256,7 +256,8 @@ class LWLink2:
                     y.features[feature.name] = feature 
 
             for x in list(response["items"][0]["payload"]["devices"].values()):
-                self.get_featureset_by_featureid(x['featureIds'][0]).product_code = x['productCode']
+                for y in x['featureIds']:
+                    self.get_featureset_by_featureid(y).product_code = x['productCode']
 
     async def async_update_featureset_states(self):
         for x in self.featuresets.values():
@@ -358,18 +359,20 @@ class LWLink2:
     # Connection
     #########################################################
 
-    async def async_connect(self, max_tries=5, _retry=0):
-        try:
-            return await self._connect_to_server()
-        except Exception as exp:
-            if (max_tries == 0) or (_retry < max_tries):
-                retry_delay = min(2 ** (_retry + 1), 120)
-                _LOGGER.warning("async_connect: Cannot connect (exception '{}'). Waiting {} seconds to retry".format(repr(exp), retry_delay))
-                await asyncio.sleep(retry_delay)
-                return await self.async_connect(max_tries, _retry + 1)
-            else:
-                _LOGGER.warning("async_connect: Cannot connect, max_tries exceeded, aborting")
-                return False
+    async def async_connect(self, max_tries=5):
+        retry_delay = 2
+        for x in range(0, max_tries):
+            try:
+                return await self._connect_to_server()
+            except Exception as exp:
+                if x < max_tries-1:
+                    _LOGGER.warning("async_connect: Cannot connect (exception '{}'). Waiting {} seconds to retry".format(repr(exp), retry_delay))
+                    await asyncio.sleep(retry_delay)
+                    retry_delay = min(2 ** (retry_delay + 1), 120)
+                else:
+                    _LOGGER.warning("async_connect: Cannot connect (exception '{}'). No more retry".format(repr(exp), retry_delay))
+        _LOGGER.warning("async_connect: Cannot connect, max_tries exceeded, aborting")
+        return False
 
     async def _connect_to_server(self):
         if (not self._websocket) or self._websocket.closed:
@@ -576,7 +579,11 @@ class LWLink2Public(LWLink2):
 
                     for z in y["features"]:
                         _LOGGER.debug("async_get_hierarchy: Adding device features {}".format(z))
-                        new_featureset.features[z["type"]] = [z["featureId"], None]
+                        feature = LWRFFeature()
+                        feature.id = z["featureId"]
+                        feature.featureset = new_featureset.featureset_id
+                        feature.name = z["type"]
+                        new_featureset.features[z["type"]] = feature
 
                     self.featuresets[y["featureSetId"]] = new_featureset
 
@@ -614,9 +621,9 @@ class LWLink2Public(LWLink2):
     async def async_update_featureset_states(self):
         feature_list = []
 
-        for dummy, x in self.featuresets.items():
-            for y in x.features:
-                feature_list.append({"featureId": x.features[y][0]})
+        for x in self.featuresets.values():
+            for y in x.features.values():
+                feature_list.append({"featureId": y.id})
 
         #split up the feature list into chunks as the public API doesn't like requests that are too long
         #200 seems safe
@@ -630,7 +637,7 @@ class LWLink2Public(LWLink2):
             for featuresetid in self.featuresets:
                 for featurename in self.featuresets[featuresetid].features:
                     if self.featuresets[featuresetid].features[featurename].id in req:
-                        self.featuresets[featuresetid].features[featurename]._state = req[self.featuresets[featuresetid].features[featurename][0]]
+                        self.featuresets[featuresetid].features[featurename]._state = req[self.featuresets[featuresetid].features[featurename].id]
 
     async def async_write_feature(self, feature_id, value):
         payload = {"value": value}
@@ -645,8 +652,8 @@ class LWLink2Public(LWLink2):
     #########################################################
 
     async def _connect_to_server(self):
-            await self._get_access_token()
-            return True
+        await self._get_access_token()
+        return True
 
 
 
